/* CHIPTEST 1.0 by ullix
 * choose any license you want
 * */

#include <esp_partition.h>
#include "FFat.h"


void partloop(esp_partition_type_t part_type) {

  esp_partition_iterator_t iterator = NULL;
  const esp_partition_t *next_partition = NULL;
  iterator = esp_partition_find(part_type, ESP_PARTITION_SUBTYPE_ANY, NULL);
  while (iterator) {
     next_partition = esp_partition_get(iterator);
     if (next_partition != NULL) {
        Serial.printf("   partition addr: 0x%06x; size: 0x%06x; label: %s\n", next_partition->address, next_partition->size, next_partition->label);
     iterator = esp_partition_next(iterator);
    }
  }
}


void listDir(fs::FS &fs, const char * dirname, uint8_t levels){

    Serial.printf("\nListing directory: %s\r\n", dirname);

    File root = fs.open(dirname);
    if(!root){
        Serial.println("- failed to open directory");
        return;
    }
    if(!root.isDirectory()){
        Serial.println(" - not a directory");
        return;
    }

    File file = root.openNextFile();
    while(file){
        if(file.isDirectory()){
            Serial.print("   DIR : ");
            Serial.println(file.name());
            if(levels){
                listDir(fs, file.name(), levels -1);
            }
        } else {
            Serial.print("   FILE: ");
            Serial.print(file.name());
            Serial.print("\tSIZE: ");
            Serial.println(file.size());
        }
        file.close();
        file = root.openNextFile();
    }
}


void ESPinfo(){

    Serial.println("\nChip Info -----------------------------------------------");

    float ESPSketchPercMax  = 100.0f * ESP.getSketchSize() / ESP.getFreeSketchSpace(); // Sketch size in percent of maxpossible sketch size

    // code
    Serial.printf("   SketchSize:     %i (0x%X) (%0.2f%% of Max)\n",    ESP.getSketchSize(), ESP.getSketchSize(), ESPSketchPercMax);
    Serial.printf("   MaxSketchSpace: %i (0x%X)\n",                     ESP.getFreeSketchSpace(), ESP.getFreeSketchSpace());
    Serial.printf("   SketchMD5:      %u\n\n",                          ESP.getSketchMD5());

    Serial.printf("   SdkVersion:     %s\n\n",                          ESP.getSdkVersion());

    // Flash
    Serial.printf("   FlashChipSize:  %i    (0x%X)\n",                  ESP.getFlashChipSize(), ESP.getFlashChipSize());
    Serial.printf("   FlashChipSpeed: %i\n",                            ESP.getFlashChipSpeed());
    Serial.printf("   FlashChipMode:  %i \n\n",                         ESP.getFlashChipMode());

    // chip stuff
    Serial.printf("   ChipID:         %u    (64bit!)\n",                ESP.getEfuseMac());     // 64 bit result
    Serial.printf("   ChipRevision:   %i\n",                            ESP.getChipRevision());
    Serial.printf("   CpuFreqMHz:     %i\n",                            ESP.getCpuFreqMHz());
    Serial.printf("   CycleCount:     %u\n\n",                          ESP.getCycleCount());

    //Internal RAM
    Serial.printf("   HeapSize:       %-7i  (0x%X)\n",                  ESP.getHeapSize(),      ESP.getHeapSize());
    Serial.printf("   FreeHeap:       %-7i  (0x%X)\n",                  ESP.getFreeHeap(),      ESP.getFreeHeap());
    Serial.printf("   MinFreeHeap:    %-7i  (0x%X)\n",                  ESP.getMinFreeHeap(),   ESP.getMinFreeHeap());
    Serial.printf("   MaxAllocHeap:   %-7i  (0x%X)\n",                  ESP.getMaxAllocHeap(),  ESP.getMaxAllocHeap());
}


void createFiles(){

    Serial.println("\nCreating some files: ");

    for (int i = 0; i < 10; i++){
        String filename = "/filename"+ String(i);
        File file = FFat.open(filename, "w");
        if(!file)   Serial.println("   Failure creating file");
        else        Serial.println(String("   Done creating file: " + filename));
        for (int j = 0; j < i * 4096; j++){
            file.write(j);
        }
        file.close();
    }
}


void setup(){
    Serial.begin(115200);
    Serial.setDebugOutput(true);

    Serial.println("\n### CHIPTEST 1.0 ##########################################");

    Serial.println("\nPartition list:");
    partloop(ESP_PARTITION_TYPE_APP);
    partloop(ESP_PARTITION_TYPE_DATA);

    Serial.println("\nTrying to mount ffat partition if present");

    bool doFormat  = true;
    //~bool doFormat  = false;

    if (doFormat) {
        Serial.println("   Formatting FFat:");
        bool formatted = FFat.format(FFAT_WIPE_FULL);
        if(!formatted){
            Serial.println("      FFat formatting failed");
        }
        else{
            Serial.println("      FFat formatting succeeded");
        }
    }

    // from FFat.h: bool begin( bool            formatOnFail   = false,
    //                          const   char *  basePath       = "/ffat",
    //                          uint8_t         maxOpenFiles   = 10,
    //                          const   char *  partitionLabel = (char*)FFAT_PARTITION_LABEL) (="ffat")
    //                        );
    bool            formatOnFail = false;
    const char *    basePath     = "/ffat";
    uint8_t         maxOpenFiles = 10;
    bool            result       = FFat.begin(formatOnFail, basePath, maxOpenFiles);
    if(!result){
        Serial.println("   FFat mount failed");
    }
    else{
        Serial.println("   File system mounted");
        Serial.printf("   Total space: %10i\n",   FFat.totalBytes());
        Serial.printf("   Free space:  %10i\n\n", FFat.freeBytes());
        createFiles();
        listDir(FFat, "/", 5);
        Serial.printf("   Free space:  %10i\n", FFat.freeBytes());
        Serial.printf("   Used space:  %10i\n", FFat.totalBytes() - FFat.freeBytes());
    }

    ESPinfo();
}


void loop(){}
